@@ -47,6 +47,7 @@ struct _NautilusSpatialWindow {
         NautilusWindow parent_object;
 
         gboolean affect_spatial_window_on_next_location_change;
+	gboolean loading;
         
         NautilusSpatialWindowDetails *details;
 };
@@ -60,6 +61,8 @@ GType            nautilus_spatial_window_get_type             (void);
 GtkWidget       *nautilus_spatial_window_get                  (const char            *uri);
 void             nautilus_spatial_window_save_geometry        (NautilusSpatialWindow *window);
 void             nautilus_spatial_window_save_scroll_position (NautilusSpatialWindow *window);
+void             nautilus_spatial_window_set_location_button  (NautilusSpatialWindow *window,
+                                                               const char            *location);
 
 
 #endif
